#!/usr/bin/env python3

import Utils

from Cheetah.Template import Template
from Cheetah.Version import VersionTuple as CheetahVersion

import datetime
import time
import os
import re
import sys
import codecs
import getopt

xml_content = Template(file="atom-feed.atom_tmpl")
xml_content.updated = time.strftime("%Y-%m-%dT%H:%M:00Z", time.gmtime(time.time()))
xml_content.program = os.path.basename(sys.argv[0])

# Default values
full = False
tls = False

def by_reverse_date(item):
    return recent_entries[item]

def usage(msg=None):
    # TODO ajouter du Tor/.onion
    sys.stderr.write ("Usage: %s [--full] [--tls]" % (sys.argv[0]))
    if msg is not None:
        sys.stderr.write (": %s" % msg)
    sys.stderr.write ("\n")
    
try:
    optlist, args = getopt.getopt (sys.argv[1:], "ft",
                                   ["full", "tls"])
    for option, value in optlist:
        if option == "--full" or option == "-f":
            full = True
        elif option == "--tls" or option == "-t":
            tls = True
        else:
            usage("Unknown option %s" % option)
            sys.exit(1)
            
except getopt.error as reason:
    print ("Error: " + sys.argv[0] + ": " + str(reason))
    usage()
    sys.exit(1)
if len(args) > 0:
    usage()
    sys.exit(1)

xml_content.entries = ""
if tls:
    if full:
        atom_extension = ".full_tls_atom"
    else:
        atom_extension = ".tls_atom"
else:
    if full:
        atom_extension = ".full_atom"
    else:
        atom_extension = ".atom"
    
recent_entries = {}
try:
    for dir in ["entries", "RFC", "ficheslec"]:
        for file in Utils.list_xml(dir):
            name = file[0]
            changed = Utils.most_recent_change(file[1])
            changed_bin = datetime.datetime.strptime(changed, "%Y-%m-%d")
            if changed_bin > datetime.datetime.now() - datetime.timedelta(days=122):
                atom = re.sub(r"\.[a-zA-Z]+_xml", atom_extension, name)
                if not os.path.exists(atom):
                    raise Exception("The file %s does not exist" % atom)
                recent_entries[atom] = changed
except:
    sys.stderr.write("Error in file %s\n" % name)
    raise
entries = list(recent_entries.keys())
entries.sort(key=by_reverse_date)
for entry in entries:
    xml_content.entries = xml_content.entries + open(entry).read()

print(xml_content.respond())
