#!/usr/bin/env python3

# TODO: rename test-consistency
# TODO: Add test for links url= with a schemeless URL (may be should be local=)

import xml.etree.ElementTree as ElementTree
import os
import sys
import re

start_dir = "."
rfc_dir = start_dir + os.sep + "RFC"
all_dirs = [rfc_dir,
            start_dir + os.sep + "ficheslec",
            start_dir + os.sep + "entries"
            ]

rfcs = {}
for rfc_file in os.listdir(rfc_dir):
    # Because of the regexp, NOT-YET files are excluded
    match = re.search(r"^([0-9]+)\.rfc_xml$", rfc_file)
    if match:
        rfcs[int(match.group(1))] = rfc_dir + os.sep + rfc_file

files = {}
for directory in all_dirs:
    for xml_file in os.listdir(directory):
        match_xml = re.search(r"_xml$", xml_file)
        excluded = re.search(r"NOT-YET", xml_file)
        if match_xml and not excluded:
            files[xml_file] = directory + os.sep + xml_file

for xml_file in list(files.keys()):
    try:
        tree = ElementTree.fromstring(open(files[xml_file]).read())
    except SyntaxError:
        print(("Wrong XML file %s" % xml_file), file=sys.stderr) 
        raise
    except UnicodeDecodeError:
        print(("Wrong Unicode encoding in %s" % xml_file), file=sys.stderr) 
        raise        
    language = tree.attrib.get('{http://www.w3.org/XML/1998/namespace}lang')
    if language is not None:
        language = language.lower()
        if language != "fr" and language[0:2] != 'fr-':
            continue # Ignore texts in other languages, since all the RFC are
              # commented in french
    rfc_elements = tree.iter("rfc")
    for rfc in rfc_elements:
        rfc_num = int(rfc.attrib["num"])
        local = "local" in rfc.attrib and \
                    (rfc.attrib["local"] == "true" or
                     rfc.attrib["local"] == 1) # TODO: RelaxNG xsd:boolean can
                     # have values 0 and 1???
        if rfc_num in list(rfcs.keys()) and \
               not local:
            match = re.search(r"^%s%s([0-9]+)\.rfc_xml$" % (rfc_dir, os.sep),
                              files[xml_file])
            if not (match and (int(match.group(1)) == rfc_num)):
                print("File %s references RFC %i without saying it is local" % \
                      (files[xml_file], rfc_num))
        else:
            if local and (rfc_num not in list(rfcs.keys())):
                print("File %s says RFC %i is local which is not true" % \
                      (files[xml_file], rfc_num))
                
